@@ -88,14 +88,14 @@ export PLATFORM_SHORT_ARCH
 export PATH=${ANDROID_NDK}/toolchains/${PATH_HOST}-${GCCVER}/prebuilt/`uname|tr A-Z a-z`-x86/bin:${PATH}
 
 # 1/ libvlc, libvlccore and its plugins
-TESTED_HASH=fe6e7e3
+TESTED_HASH=45adae3485
 if [ ! -d "vlc" ]; then
     echo "VLC source not found, cloning"
     git clone git://git.videolan.org/vlc.git vlc
     cd vlc
     git checkout -B android ${TESTED_HASH}
     echo "Applying the patches"
-    git am ../patches/*.patch
+    git am -3 ../patches/*.patch
     if [ $? -ne 0 ]; then
         git am --abort
         echo "Applying the patches failed, aborting git-am"
