@@ -35,10 +35,12 @@ import org.videolan.vlc.interfaces.IAudioService;
 import org.videolan.vlc.interfaces.IAudioServiceCallback;
 
 import android.annotation.SuppressLint;
+import android.annotation.TargetApi;
 import android.app.Notification;
 import android.app.PendingIntent;
 import android.app.Service;
 import android.content.BroadcastReceiver;
+import android.content.ComponentName;
 import android.content.ContentResolver;
 import android.content.Context;
 import android.content.Intent;
@@ -47,6 +49,10 @@ import android.database.Cursor;
 import android.graphics.Bitmap;
 import android.graphics.BitmapFactory;
 import android.media.AudioManager;
+import android.media.AudioManager.OnAudioFocusChangeListener;
+import android.media.MediaMetadataRetriever;
+import android.media.RemoteControlClient;
+import android.media.RemoteControlClient.MetadataEditor;
 import android.net.Uri;
 import android.os.Handler;
 import android.os.IBinder;
@@ -86,6 +92,19 @@ public class AudioService extends Service {
     private long mHeadsetDownTime = 0;
     private long mHeadsetUpTime = 0;
 
+    /**
+     * RemoteControlClient is for lock screen playback control.
+     */
+    private RemoteControlClient mRemoteControlClient = null;
+
+    /**
+     * Distinguish between the "fake" (Java-backed) playlist versus the "real"
+     * (LibVLC/LibVLCcore) backed playlist.
+     *
+     * True if being backed by LibVLC, false if "virtually" backed by Java.
+     */
+    private boolean mLibVLCPlaylistActive = false;
+
     @Override
     public void onCreate() {
         super.onCreate();
@@ -103,7 +122,7 @@ public class AudioService extends Service {
         mCallback = new HashMap<IAudioServiceCallback, Integer>();
         mMediaList = new ArrayList<Media>();
         mPrevious = new Stack<Media>();
-        mEventManager = EventManager.getIntance();
+        mEventManager = EventManager.getInstance();
 
         IntentFilter filter = new IntentFilter();
         filter.setPriority(Integer.MAX_VALUE);
@@ -115,6 +134,48 @@ public class AudioService extends Service {
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
         filter.addAction(Intent.ACTION_MEDIA_BUTTON);
         registerReceiver(serviceReceiver, filter);
+
+        if(Util.isICSOrLater())
+            setUpRemoteControlClient();
+    }
+
+    @TargetApi(14)
+    private void setUpRemoteControlClient() {
+        if(!Util.isICSOrLater()) // sanity check
+            return;
+
+        AudioManager audioManager = (AudioManager)getSystemService(AUDIO_SERVICE);
+        ComponentName eventReceiverComponent = new ComponentName(getPackageName(), RemoteControlClientReceiver.class.getName());
+
+        audioManager.registerMediaButtonEventReceiver(eventReceiverComponent);
+
+        Intent mediaButtonIntent = new Intent(Intent.ACTION_MEDIA_BUTTON);
+        mediaButtonIntent.setComponent(eventReceiverComponent);
+        PendingIntent mediaPendingIntent = PendingIntent.getBroadcast(getApplicationContext(), 0, mediaButtonIntent, 0);
+
+        // create and register the remote control client
+        mRemoteControlClient = new RemoteControlClient(mediaPendingIntent);
+        mRemoteControlClient.setTransportControlFlags(
+                RemoteControlClient.FLAG_KEY_MEDIA_PLAY |
+                RemoteControlClient.FLAG_KEY_MEDIA_PAUSE |
+                RemoteControlClient.FLAG_KEY_MEDIA_PREVIOUS |
+                RemoteControlClient.FLAG_KEY_MEDIA_NEXT |
+                RemoteControlClient.FLAG_KEY_MEDIA_STOP);
+        audioManager.registerRemoteControlClient(mRemoteControlClient);
+    }
+
+    /**
+     * A function to control the Remote Control Client. It is needed for
+     * compatibility with devices below Ice Cream Sandwich (4.0).
+     *
+     * @param p Playback state
+     */
+    @TargetApi(14)
+    private void setRemoteControlClientPlaybackState(int p) {
+        if(!Util.isICSOrLater())
+            return;
+
+        mRemoteControlClient.setPlaybackState(p);
     }
 
     @Override
@@ -134,6 +195,41 @@ public class AudioService extends Service {
         return mInterface;
     }
 
+    @TargetApi(8)
+    private void changeAudioFocus(boolean gain) {
+        if(!Util.isGingerbreadOrLater()) // NOP if not supported
+            return;
+
+        AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
+        if(gain)
+            am.requestAudioFocus(audioFocusListener, AudioManager.STREAM_MUSIC, AudioManager.AUDIOFOCUS_GAIN);
+        else
+            am.abandonAudioFocus(audioFocusListener);
+    }
+
+    private final OnAudioFocusChangeListener audioFocusListener = new OnAudioFocusChangeListener() {
+        int volume = -1;
+        @Override
+        public void onAudioFocusChange(int focusChange) {
+            AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
+            if(focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT_CAN_DUCK ||
+               focusChange == AudioManager.AUDIOFOCUS_LOSS_TRANSIENT) {
+                volume = am.getStreamVolume(AudioManager.STREAM_MUSIC);
+                int maxVol = am.getStreamMaxVolume(AudioManager.STREAM_MUSIC);
+                /*
+                 * Lower the volume to 19% to "duck" when an alert or something
+                 * needs to be played.
+                 */
+                am.setStreamVolume(AudioManager.STREAM_MUSIC, (int)(0.19*maxVol), 0);
+            } else {
+                if(volume != -1) {
+                    am.setStreamVolume(AudioManager.STREAM_MUSIC, volume, 0);
+                    volume = -1;
+                }
+            }
+        }
+    };
+
     private final BroadcastReceiver serviceReceiver = new BroadcastReceiver() {
         @Override
         public void onReceive(Context context, Intent intent) {
@@ -269,19 +365,25 @@ public class AudioService extends Service {
         @Override
         public void handleMessage(Message msg) {
             AudioService service = getOwner();
+            if(service == null) return;
+
             switch (msg.getData().getInt("event")) {
                 case EventManager.MediaPlayerPlaying:
                     Log.i(TAG, "MediaPlayerPlaying");
+                    service.changeAudioFocus(true);
+                    service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PLAYING);
                     break;
                 case EventManager.MediaPlayerPaused:
                     Log.i(TAG, "MediaPlayerPaused");
                     service.executeUpdate();
                     // also hide notification if phone ringing
                     service.hideNotification();
+                    service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_PAUSED);
                     break;
                 case EventManager.MediaPlayerStopped:
                     Log.i(TAG, "MediaPlayerStopped");
                     service.executeUpdate();
+                    service.setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
                     break;
                 case EventManager.MediaPlayerEndReached:
                     Log.i(TAG, "MediaPlayerEndReached");
@@ -305,13 +407,11 @@ public class AudioService extends Service {
         mMediaList.clear();
         hideNotification();
 
-        // Got video, switch to the video player
-        Intent intent = new Intent(VLCApplication.getAppContext(), VideoPlayerActivity.class);
-        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_MULTIPLE_TASK);
-        intent.putExtra("itemLocation", mCurrentMedia.getLocation());
-        // Don't lose the currently playing stream
-        intent.putExtra("dontParse", true);
-        startActivity(intent);
+        // Don't crash if user stopped the media
+        if(mCurrentMedia == null) return;
+
+        // Switch to the video player & don't lose the currently playing stream
+        VideoPlayerActivity.start(VLCApplication.getAppContext(), mCurrentMedia.getLocation(), mCurrentMedia.getTitle(), true);
     }
 
     private void executeUpdate() {
@@ -340,6 +440,8 @@ public class AudioService extends Service {
         @Override
         public void handleMessage(Message msg) {
             AudioService service = getOwner();
+            if(service == null) return;
+
             switch (msg.what) {
                 case SHOW_PROGRESS:
                     if (service.mCallback.size() > 0) {
@@ -392,12 +494,14 @@ public class AudioService extends Service {
     private void stop() {
         mEventManager.removeHandler(mEventHandler);
         mLibVLC.stop();
+        setRemoteControlClientPlaybackState(RemoteControlClient.PLAYSTATE_STOPPED);
         mCurrentMedia = null;
         mMediaList.clear();
         mPrevious.clear();
         mHandler.removeMessages(SHOW_PROGRESS);
         hideNotification();
         executeUpdate();
+        changeAudioFocus(false);
     }
 
     private void next() {
@@ -419,10 +523,35 @@ public class AudioService extends Service {
                 return;
             }
         }
-        mLibVLC.readMedia(mCurrentMedia.getLocation(), true);
+        if(mLibVLCPlaylistActive) {
+            if(mRepeating == RepeatType.None)
+                mLibVLC.next();
+            else if(mRepeating == RepeatType.Once)
+                mLibVLC.playIndex(index);
+            else
+                mLibVLC.playIndex(mMediaList.indexOf(mCurrentMedia));
+        } else {
+            mLibVLC.readMedia(mCurrentMedia.getLocation(), true);
+        }
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
         showNotification();
         updateWidget(this);
+        updateRemoteControlClientMetadata();
+    }
+
+    @TargetApi(14)
+    private void updateRemoteControlClientMetadata() {
+        if(!Util.isICSOrLater()) // NOP check
+            return;
+
+        MetadataEditor editor = mRemoteControlClient.editMetadata(true);
+        editor.putString(MediaMetadataRetriever.METADATA_KEY_ALBUM, mCurrentMedia.getAlbum());
+        editor.putString(MediaMetadataRetriever.METADATA_KEY_ARTIST, mCurrentMedia.getArtist());
+        editor.putString(MediaMetadataRetriever.METADATA_KEY_GENRE, mCurrentMedia.getGenre());
+        editor.putString(MediaMetadataRetriever.METADATA_KEY_TITLE, mCurrentMedia.getTitle());
+        editor.putLong(MediaMetadataRetriever.METADATA_KEY_DURATION, mCurrentMedia.getLength());
+        editor.putBitmap(MetadataEditor.BITMAP_KEY_ARTWORK, getCover());
+        editor.apply();
     }
 
     private void previous() {
@@ -433,10 +562,20 @@ public class AudioService extends Service {
             mCurrentMedia = mMediaList.get(index - 1);
         else
             return;
-        mLibVLC.readMedia(mCurrentMedia.getLocation(), true);
+        if(mLibVLCPlaylistActive) {
+            if(mRepeating == RepeatType.None)
+                mLibVLC.previous();
+            else if(mRepeating == RepeatType.Once)
+                mLibVLC.playIndex(index);
+            else
+                mLibVLC.playIndex(mMediaList.indexOf(mCurrentMedia));
+        } else {
+            mLibVLC.readMedia(mCurrentMedia.getLocation(), true);
+        }
         mHandler.sendEmptyMessage(SHOW_PROGRESS);
         showNotification();
         updateWidget(this);
+        updateRemoteControlClientMetadata();
     }
 
     private void shuffle() {
@@ -469,9 +608,11 @@ public class AudioService extends Service {
                 int titleColumn = cursor.getColumnIndex(android.provider.MediaStore.Audio.Albums.ALBUM_ART);
                 String albumArt = cursor.getString(titleColumn);
                 cursor.close();
-                Bitmap b = BitmapFactory.decodeFile(albumArt);
-                if (b != null)
-                    return b;
+                if(albumArt != null) { // could be null (no album art stored)
+                    Bitmap b = BitmapFactory.decodeFile(albumArt);
+                    if (b != null)
+                        return b;
+                }
             }
 
             //cover not in MediaStore, trying vlc
@@ -628,21 +769,30 @@ public class AudioService extends Service {
         }
 
         @Override
-        public void load(List<String> mediaPathList, int position)
+        public void load(List<String> mediaPathList, int position, boolean libvlcBacked)
                 throws RemoteException {
+            mLibVLCPlaylistActive = libvlcBacked;
+
             Log.v(TAG, "Loading position " + ((Integer)position).toString() + " in " + mediaPathList.toString());
             mEventManager.addHandler(mEventHandler);
+
             mMediaList.clear();
             mPrevious.clear();
-            DatabaseManager db = DatabaseManager.getInstance(AudioService.this);
-            for (int i = 0; i < mediaPathList.size(); i++) {
-                String path = mediaPathList.get(i);
-                Media media = db.getMedia(AudioService.this, path);
-                if(media == null) {
-                    Log.v(TAG, "Creating on-the-fly Media object for " + path);
-                    media = new Media(path, false);
+
+            if(mLibVLCPlaylistActive) {
+                for(int i = 0; i < mediaPathList.size(); i++)
+                    mMediaList.add(new Media(mediaPathList.get(i), i));
+            } else {
+                DatabaseManager db = DatabaseManager.getInstance(AudioService.this);
+                for (int i = 0; i < mediaPathList.size(); i++) {
+                    String path = mediaPathList.get(i);
+                    Media media = db.getMedia(AudioService.this, path);
+                    if(media == null) {
+                        Log.v(TAG, "Creating on-the-fly Media object for " + path);
+                        media = new Media(path, false);
+                    }
+                    mMediaList.add(media);
                 }
-                mMediaList.add(media);
             }
 
             if (mMediaList.size() > position) {
@@ -650,9 +800,14 @@ public class AudioService extends Service {
             }
 
             if (mCurrentMedia != null) {
-                mLibVLC.readMedia(mCurrentMedia.getLocation());
+                if(mLibVLCPlaylistActive) {
+                    mLibVLC.playIndex(position);
+                } else {
+                    mLibVLC.readMedia(mCurrentMedia.getLocation());
+                }
                 showNotification();
                 updateWidget(AudioService.this);
+                updateRemoteControlClientMetadata();
             }
         }
 
@@ -692,14 +847,21 @@ public class AudioService extends Service {
         @Override
         public void append(List<String> mediaPathList) throws RemoteException {
             if (mMediaList.size() == 0) {
-                load(mediaPathList, 0);
+                load(mediaPathList, 0, false);
                 return;
             }
 
+            if(mLibVLCPlaylistActive) {
+                return;
+            }
             DatabaseManager db = DatabaseManager.getInstance(AudioService.this);
             for (int i = 0; i < mediaPathList.size(); i++) {
                 String path = mediaPathList.get(i);
                 Media media = db.getMedia(AudioService.this, path);
+                if(media == null) {
+                    Log.v(TAG, "Creating on-the-fly Media object for " + path);
+                    media = new Media(path, false);
+                }
                 mMediaList.add(media);
             }
         }
