@@ -32,12 +32,14 @@ import java.util.Stack;
 
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.audio.AudioBrowserFragment;
+import org.videolan.vlc.gui.video.VideoListFragment;
 
 import android.content.Context;
 import android.content.SharedPreferences;
 import android.os.Environment;
 import android.os.Handler;
 import android.preference.PreferenceManager;
+import android.util.Log;
 
 public class MediaLibrary {
     public final static String TAG = "VLC/MediaLibrary";
@@ -48,6 +50,7 @@ public class MediaLibrary {
     private final DatabaseManager mDBManager;
     private final ArrayList<Media> mItemList;
     private final ArrayList<Handler> mUpdateHandler;
+    private boolean isStopping = false;
     protected Thread mLoadingThread;
 
     private MediaLibrary(Context context) {
@@ -59,11 +62,17 @@ public class MediaLibrary {
 
     public void loadMediaItems(Context context) {
         if (mLoadingThread == null || mLoadingThread.getState() == State.TERMINATED) {
+            isStopping = false;
+            VideoListFragment.actionScanStart(context.getApplicationContext());
             mLoadingThread = new Thread(new GetMediaItemsRunnable(context.getApplicationContext()));
             mLoadingThread.start();
         }
     }
 
+    public void stop() {
+        isStopping = true;
+    }
+
     public static MediaLibrary getInstance(Context context) {
         if (mInstance == null)
             mInstance = new MediaLibrary(context);
@@ -151,6 +160,10 @@ public class MediaLibrary {
         return items;
     }
 
+    public boolean ismLoadingThreadrunning () {
+        return mLoadingThread.isAlive();
+    };
+
     private class GetMediaItemsRunnable implements Runnable {
 
         private final Stack<File> directories = new Stack<File>();
@@ -198,89 +211,108 @@ public class MediaLibrary {
             int count = 0;
 
             ArrayList<File> mediaToScan = new ArrayList<File>();
+            try {
+                // Count total files, and stack them
+                while (!directories.isEmpty()) {
+                    File dir = directories.pop();
+                    String dirPath = dir.getAbsolutePath();
+                    File[] f = null;
 
-            // Count total files, and stack them
-            while (!directories.isEmpty()) {
-                File dir = directories.pop();
-                String dirPath = dir.getAbsolutePath();
-                File[] f = null;
+                    // Skip some system folders
+                    if (dirPath.startsWith("/proc/") || dirPath.startsWith("/sys/") || dirPath.startsWith("/dev/"))
+                        continue;
 
-                // Skip some system folders
-                if (dirPath.startsWith("/proc/") || dirPath.startsWith("/sys/") || dirPath.startsWith("/dev/"))
-                    continue;
+                    // Do not scan again if same canonical path
+                    try {
+                        dirPath = dir.getCanonicalPath();
+                    } catch (IOException e) {
+                        e.printStackTrace();
+                    }
+                    if (directoriesScanned.contains(dirPath))
+                        continue;
+                    else
+                        directoriesScanned.add(dirPath);
 
-                // Do not scan again if same canonical path
-                try {
-                    dirPath = dir.getCanonicalPath();
-                } catch (IOException e) {
-                    e.printStackTrace();
-                }
-                if (directoriesScanned.contains(dirPath))
-                    continue;
-                else
-                    directoriesScanned.add(dirPath);
+                    // Do no scan media in .nomedia folders
+                    if (new File(dirPath + "/.nomedia").exists()) {
+                        continue;
+                    }
 
-                // Do no scan media in .nomedia folders
-                if (new File(dirPath + "/.nomedia").exists()) {
-                    continue;
-                }
-
-                // Filter the extensions and the folders
-                if ((f = dir.listFiles(mediaFileFilter)) != null) {
-                    for (File file : f) {
-                        if (file.isFile()) {
-                            mediaToScan.add(file);
-                        } else if (file.isDirectory()) {
-                            directories.push(file);
+                    // Filter the extensions and the folders
+                    try { 
+                        if ((f = dir.listFiles(mediaFileFilter)) != null) {
+                            for (File file : f) {
+                                if (file.isFile()) {
+                                    mediaToScan.add(file);
+                                } else if (file.isDirectory()) {
+                                    directories.push(file);
+                                }
+                            }
                         }
+                    } catch (Exception e)
+                    {
+                        // listFiles can fail in OutOfMemoryError, go to the next folder
+                        continue;
+                    }
+
+                    if (isStopping) {
+                        Log.d(TAG, "Stopping scan");
+                        return;
                     }
                 }
-            }
 
-            // Process the stacked items
-            for (File file : mediaToScan) {
-                String fileURI = Util.PathToURI(file.getPath());
-                MainActivity.sendTextInfo(mContext, file.getName(), count,
-                        mediaToScan.size());
-                count++;
-                if (existingMedias.containsKey(fileURI)) {
-                    /**
-                     * only add file if it is not already in the list. eg. if
-                     * user select an subfolder as well
-                     */
-                    if (!addedLocations.contains(fileURI)) {
-                        // get existing media item from database
-                        mItemList.add(existingMedias.get(fileURI));
-                        addedLocations.add(fileURI);
+                // Process the stacked items
+                for (File file : mediaToScan) {
+                    String fileURI = Util.PathToURI(file.getPath());
+                    MainActivity.sendTextInfo(mContext, file.getName(), count,
+                            mediaToScan.size());
+                    count++;
+                    if (existingMedias.containsKey(fileURI)) {
+                        /**
+                         * only add file if it is not already in the list. eg. if
+                         * user select an subfolder as well
+                         */
+                        if (!addedLocations.contains(fileURI)) {
+                            // get existing media item from database
+                            mItemList.add(existingMedias.get(fileURI));
+                            addedLocations.add(fileURI);
+                        }
+                    } else {
+                        // create new media item
+                        mItemList.add(new Media(fileURI, true));
+                    }
+                    if (isStopping) {
+                        Log.d(TAG, "Stopping scan");
+                        return;
                     }
-                } else {
-                    // create new media item
-                    mItemList.add(new Media(fileURI, true));
                 }
-            }
-
-            // update the video and audio activities
-            for (int i = 0; i < mUpdateHandler.size(); i++) {
-                Handler h = mUpdateHandler.get(i);
-                h.sendEmptyMessage(MEDIA_ITEMS_UPDATED);
-            }
-
-            // remove old files & folders from database if storage is mounted
-            if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
-                for (String fileURI : addedLocations) {
-                    existingMedias.remove(fileURI);
+            } finally {
+                // update the video and audio activities
+                for (int i = 0; i < mUpdateHandler.size(); i++) {
+                    Handler h = mUpdateHandler.get(i);
+                    h.sendEmptyMessage(MEDIA_ITEMS_UPDATED);
                 }
-                mDBManager.removeMedias(existingMedias.keySet());
 
-                for (File file : mDBManager.getMediaDirs())
-                    if (!file.isDirectory())
-                        mDBManager.removeDir(file.getAbsolutePath());
+                // remove old files & folders from database if storage is mounted
+                if (Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)) {
+                    for (String fileURI : addedLocations) {
+                        existingMedias.remove(fileURI);
+                    }
+                    mDBManager.removeMedias(existingMedias.keySet());
+
+                    for (File file : mDBManager.getMediaDirs())
+                        if (!file.isDirectory())
+                            mDBManager.removeDir(file.getAbsolutePath());
+                }
+
+                // hide progressbar in footer
+                MainActivity.clearTextInfo(mContext);
+                MainActivity.hideProgressBar(mContext);
+
+                VideoListFragment.actionScanStop(mContext);
+
+                mContext = null;
             }
-
-            // hide progressbar in footer
-            MainActivity.clearTextInfo(mContext);
-            MainActivity.hideProgressBar(mContext);
-            mContext = null;
         }
     };
 
