@@ -28,18 +28,36 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.text.DecimalFormat;
+import java.text.NumberFormat;
+import java.util.HashSet;
+import java.util.Locale;
 import java.util.Properties;
 
 import android.content.Context;
 import android.graphics.Bitmap;
 import android.net.Uri;
+import android.telephony.TelephonyManager;
+import android.util.DisplayMetrics;
+import android.util.TypedValue;
+import android.view.Display;
 import android.view.View;
+import android.view.WindowManager;
 import android.widget.Toast;
 
 public class Util {
     public final static String TAG = "VLC/Util";
+    public final static boolean hasNavBar;
     /** A set of utility functions for the VLC application */
 
+    static {
+        HashSet<String> devicesWithoutNavBar = new HashSet<String>();
+        devicesWithoutNavBar.add("HTC One V");
+        devicesWithoutNavBar.add("HTC One S");
+        devicesWithoutNavBar.add("HTC One X");
+        devicesWithoutNavBar.add("HTC One XL");
+        hasNavBar = isICSOrLater() && !devicesWithoutNavBar.contains(android.os.Build.MODEL);
+    }
+
     /** Print an on-screen message to alert the user */
     public static void toaster(Context context, int stringId, int duration) {
         Toast.makeText(context, stringId, duration).show();
@@ -56,7 +74,8 @@ public class Util {
     public static String URItoFileName(String URI) {
         int sep = URI.lastIndexOf('/');
         int dot = URI.lastIndexOf('.');
-        return Uri.decode(URI.substring(sep + 1, dot));
+        String name = dot >= 0 ? URI.substring(sep + 1, dot) : URI;
+        return Uri.decode(name);
     }
 
     public static String PathToURI(String path) {
@@ -114,7 +133,8 @@ public class Util {
         int hours = (int) millis;
 
         String time;
-        DecimalFormat format = new DecimalFormat("00");
+        DecimalFormat format = (DecimalFormat)NumberFormat.getInstance(Locale.US);
+        format.applyPattern("00");
         if (millis > 0) {
             time = (negative ? "-" : "") + hours + ":" + format.format(min) + ":" + format.format(sec);
         } else {
@@ -179,6 +199,24 @@ public class Util {
                 : R.drawable.background_item2);
     }
 
+    public static int convertPxToDp(int px) {
+        WindowManager wm = (WindowManager)VLCApplication.getAppContext().
+                getSystemService(Context.WINDOW_SERVICE);
+        Display display = wm.getDefaultDisplay();
+        DisplayMetrics metrics = new DisplayMetrics();
+        display.getMetrics(metrics);
+        float logicalDensity = metrics.density;
+        int dp = Math.round(px / logicalDensity);
+        return dp;
+    }
+
+    public static int convertDpToPx(int dp) {
+        return Math.round(
+                TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_DIP, dp,
+                VLCApplication.getAppResources().getDisplayMetrics())
+                         );
+    }
+
     public static boolean isGingerbreadOrLater()
     {
         return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.GINGERBREAD;
@@ -194,6 +232,11 @@ public class Util {
         return android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.ICE_CREAM_SANDWICH;
     }
 
+    public static boolean hasNavBar()
+    {
+        return hasNavBar;
+    }
+
     private static String errorMsg = null;
     private static boolean isCompatible = false;
     public static String getErrorMsg() {
@@ -215,25 +258,50 @@ public class Util {
             return false;
         }
 
+        String CPU_ABI = android.os.Build.CPU_ABI;
+        String CPU_ABI2 = "none";
+        if(android.os.Build.VERSION.SDK_INT >= 8) { // CPU_ABI2 since 2.2
+            try {
+                CPU_ABI2 = (String)android.os.Build.class.getDeclaredField("CPU_ABI2").get(null);
+            } catch (Exception e) { }
+        }
+
         String ANDROID_ABI = properties.getProperty("ANDROID_ABI");
-        boolean NO_NEON = properties.getProperty("NO_NEON").equals("1");
-        boolean NO_FPU = properties.getProperty("NO_FPU").equals("1");
-        boolean NO_ARMV6 = properties.getProperty("NO_ARMV6").equals("1");
+        boolean NO_NEON = properties.getProperty("NO_NEON","0").equals("1");
+        boolean NO_FPU = properties.getProperty("NO_FPU","0").equals("1");
+        boolean NO_ARMV6 = properties.getProperty("NO_ARMV6","0").equals("1");
         boolean hasNeon = false, hasFpu = false, hasArmV6 = false, hasArmV7 = false;
+        boolean hasX86 = false;
+
+        if(CPU_ABI.equals("x86")) {
+            hasX86 = true;
+        } else if(CPU_ABI.equals("armeabi-v7a") ||
+                  CPU_ABI2.equals("armeabi-v7a")) {
+            hasArmV7 = true;
+            hasArmV6 = true; /* Armv7 is backwards compatible to < v6 */
+        } else if(CPU_ABI.equals("armeabi") ||
+                  CPU_ABI2.equals("armeabi")) {
+            hasArmV6 = true;
+        }
 
         try {
             FileReader fileReader = new FileReader("/proc/cpuinfo");
             BufferedReader br = new BufferedReader(fileReader);
             String line;
             while((line = br.readLine()) != null) {
-                if(!hasNeon && line.contains("neon"))
-                    hasNeon = true;
                 if(!hasArmV7 && line.contains("ARMv7")) {
                     hasArmV7 = true;
                     hasArmV6 = true; /* Armv7 is backwards compatible to < v6 */
                 }
                 if(!hasArmV7 && !hasArmV6 && line.contains("ARMv6"))
                     hasArmV6 = true;
+                // "clflush size" is a x86-specific cpuinfo tag.
+                // (see kernel sources arch/x86/kernel/cpu/proc.c)
+                if(line.contains("clflush size"))
+                    hasX86 = true;
+                // TODO: MIPS - "microsecond timers"; see arch/mips/kernel/proc.c
+                if(!hasNeon && line.contains("neon"))
+                    hasNeon = true;
                 if(!hasFpu && line.contains("vfp"))
                     hasFpu = true;
             }
@@ -245,6 +313,17 @@ public class Util {
             return false;
         }
 
+        // Enforce proper architecture to prevent problems
+        if(ANDROID_ABI.equals("x86") && !hasX86) {
+            errorMsg = "x86 build on non-x86 device";
+            isCompatible = false;
+            return false;
+        } else if(hasX86 && ANDROID_ABI.contains("armeabi")) {
+            errorMsg = "ARM build on x86 device";
+            isCompatible = false;
+            return false;
+        }
+
         if(ANDROID_ABI.equals("armeabi-v7a") && !hasArmV7) {
             errorMsg = "ARMv7 build on non-ARMv7 device";
             isCompatible = false;
@@ -261,13 +340,24 @@ public class Util {
                 return false;
             }
         }
-        if(!NO_NEON && !hasNeon) {
-            errorMsg = "NEON build on non-NEON device";
-            isCompatible = false;
-            return false;
+        if(ANDROID_ABI.equals("armeabi") || ANDROID_ABI.equals("armeabi-v7a")) {
+            if(!NO_NEON && !hasNeon) {
+                errorMsg = "NEON build on non-NEON device";
+                isCompatible = false;
+                return false;
+            }
         }
         errorMsg = null;
         isCompatible = true;
         return true;
     }
+
+    public static boolean isPhone(){
+        TelephonyManager manager = (TelephonyManager)VLCApplication.getAppContext().getSystemService(Context.TELEPHONY_SERVICE);
+        if(manager.getPhoneType() == TelephonyManager.PHONE_TYPE_NONE){
+            return false;
+        }else{
+            return true;
+        }
+    }
 }
