@@ -124,13 +124,13 @@ public class MainActivity extends SherlockFragmentActivity {
         updateMenuOffset();
 
         View v_main = LayoutInflater.from(this).inflate(R.layout.main, null);
-        mMenu.setViewAbove(v_main);
+        mMenu.setContent(v_main);
         View sidebar = LayoutInflater.from(this).inflate(R.layout.sidebar, null);
         ((ListView)sidebar).setFooterDividersEnabled(true);
         final ListView listView = (ListView)sidebar.findViewById(android.R.id.list);
         mSidebarAdapter = new SidebarAdapter(getSupportFragmentManager());
         listView.setAdapter(mSidebarAdapter);
-        mMenu.setViewBehind(sidebar);
+        mMenu.setMenu(sidebar);
 
         /* Get the current version from package */
         PackageInfo pinfo = null;
@@ -184,7 +184,7 @@ public class MainActivity extends SherlockFragmentActivity {
         /* Add padding between the home button and the arrow */
         ImageView home = (ImageView)findViewById(Util.isHoneycombOrLater()
                 ? android.R.id.home : R.id.abs__home);
-        if (home != null) 
+        if (home != null)
             home.setPadding(20, 0, 0, 0);
 
         /* Set up the sidebar click listener */
@@ -195,8 +195,11 @@ public class MainActivity extends SherlockFragmentActivity {
                     int position, long id) {
                 SidebarAdapter.SidebarEntry entry = (SidebarEntry) listView.getItemAtPosition(position);
                 Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
-                if(current.getTag() == entry.id) /* Already selected */
+
+                if(current == null || current.getTag() == entry.id) { /* Already selected */
+                    mMenu.showAbove();
                     return;
+                }
 
                 /* Clear any backstack before switching tabs.
                  * This way it's more consistent for the user, who might have
@@ -307,6 +310,9 @@ public class MainActivity extends SherlockFragmentActivity {
      */
     @Override
     protected void onPause() {
+        /* Stop scanning for files */
+        MediaLibrary.getInstance(this).stop();
+
         SharedPreferences.Editor editor = getSharedPreferences("MainActivity", MODE_PRIVATE).edit();
         editor.putString("fragment", mCurrentFragment);
         editor.commit();
