@@ -20,9 +20,6 @@
 
 package org.videolan.vlc.gui.audio;
 
-import java.io.File;
-import java.net.URI;
-import java.net.URISyntaxException;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
@@ -31,10 +28,12 @@ import org.videolan.vlc.AudioServiceController;
 import org.videolan.vlc.Media;
 import org.videolan.vlc.MediaLibrary;
 import org.videolan.vlc.R;
+import org.videolan.vlc.Util;
+import org.videolan.vlc.VlcRunnable;
 import org.videolan.vlc.WeakHandler;
+import org.videolan.vlc.gui.CommonDialogs;
 
 import android.app.AlertDialog;
-import android.content.DialogInterface;
 import android.content.Intent;
 import android.os.Bundle;
 import android.os.Handler;
@@ -122,46 +121,24 @@ public class AudioListFragment extends SherlockListFragment {
     @Override
     public void onListItemClick(ListView l, View v, int position, long id) {
         mAudioController.load(mSongsAdapter.getLocations(), position);
-        Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-        startActivity(intent);
+        AudioPlayerActivity.start(getActivity());
         super.onListItemClick(l, v, position, id);
     }
 
-    public void deleteMedia( final List<String> addressMedia, final Media aMedia ) {
-        AlertDialog alertDialog = new AlertDialog.Builder(getActivity())
-        .setTitle(R.string.confirm_delete)
-        .setMessage(R.string.validation)
-        .setIcon(android.R.drawable.ic_dialog_alert)
-        .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
-            public void onClick(DialogInterface dialog, int whichButton) {
-                URI adressMediaUri = null;
-                try {
-                    adressMediaUri = new URI (addressMedia.get(0));
-                } catch (URISyntaxException e) {
-                    // TODO Auto-generated catch block
-                    e.printStackTrace();
-                }
-                File fileMedia =  new File(adressMediaUri);
-                fileMedia.delete();
-                mMediaLibrary.getMediaItems().remove(aMedia);
-                updateList();
-            }
-        })
-        .setNegativeButton(android.R.string.cancel, null).create();
-        
-        alertDialog.show();
-    }
-
     @Override
     public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
         MenuInflater inflater = getActivity().getMenuInflater();
         inflater.inflate(R.menu.audio_list_browser, menu);
+        if (!Util.isPhone())
+            menu.setGroupVisible(R.id.phone_only, false);
     }
 
     @Override
     public boolean onContextItemSelected(MenuItem item) {
         AdapterContextMenuInfo menuInfo = (AdapterContextMenuInfo) item.getMenuInfo();
+        if(menuInfo == null) // getMenuInfo can be NULL
+            return super.onContextItemSelected(item);
+
         int startPosition;
         List<String> medias;
         int id = item.getItemId();
@@ -172,8 +149,23 @@ public class AudioListFragment extends SherlockListFragment {
                           id == R.id.audio_list_browser_append_all);
 
         if (id == R.id.audio_list_browser_delete) {
-            deleteMedia(mSongsAdapter.getLocation(menuInfo.position),
-                        mSongsAdapter.getItem(menuInfo.position));
+            final Media media = mSongsAdapter.getItem(menuInfo.position);
+            AlertDialog dialog = CommonDialogs.deleteMedia(
+                    getActivity(),
+                    media.getLocation(),
+                    new VlcRunnable(media) {
+                        @Override
+                        public void run(Object o) {
+                            mMediaLibrary.getMediaItems().remove(media);
+                            updateList();
+                        }
+                    });
+            dialog.show();
+            return true;
+        }
+
+        if (id == R.id.audio_list_browser_set_song) {
+            AudioUtil.setRingtone(mSongsAdapter.getItem(menuInfo.position),getActivity());
             return true;
         }
 
@@ -190,9 +182,7 @@ public class AudioListFragment extends SherlockListFragment {
         else
             mAudioController.load(medias, startPosition);
 
-        Intent intent = new Intent(getActivity(), AudioPlayerActivity.class);
-        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
-        startActivity(intent);
+        AudioPlayerActivity.start(getActivity());
         return super.onContextItemSelected(item);
     }
 
@@ -209,6 +199,8 @@ public class AudioListFragment extends SherlockListFragment {
         @Override
         public void handleMessage(Message msg) {
             AudioListFragment fragment = getOwner();
+            if(fragment == null) return;
+
             switch (msg.what) {
                 case MediaLibrary.MEDIA_ITEMS_UPDATED:
                     fragment.updateList();
