@@ -211,6 +211,11 @@ public class VideoPlayerActivity extends Activity {
             @Override
             public void onScrollStarted() {
                 showOverlay(OVERLAY_INFINITE);
+                /*FIXME
+                 * The setTracksAndSubtitles function probably doesn't work in case of many many Tracks and Subtitles
+                 * Moreover, in a video flux, if Tracks & Subtitles change, they won't be updated
+                 */
+                setTracksAndSubtitles();
             }
 
             @Override
@@ -1086,38 +1091,6 @@ public class VideoPlayerActivity extends Activity {
             mHandler.removeMessages(FADE_OUT);
             mHandler.sendMessageDelayed(msg, timeout);
         }
-        if (mAudioTracks == null) {
-            mAudioTracks = mLibVLC.getAudioTrackDescription();
-            if (mAudioTracks != null && mAudioTracks.length > 1) {
-                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mAudioTracks);
-                mAudio.setAdapter(adapter);
-                int current = mLibVLC.getAudioTrack() - 1;
-                mAudio.setSelection(current);
-                mAudio.setOnItemSelectedListener(mAudioListener);
-                mAudio.setEnabled(true);
-            }
-            else {
-                mAudio.setEnabled(false);
-                mAudio.setOnItemSelectedListener(null);
-                mAudio.setAdapter(null);
-            }
-        }
-        if (mSubtitleTracks == null) {
-            mSubtitleTracks = mLibVLC.getSpuTrackDescription();
-            if (mSubtitleTracks != null && mSubtitleTracks.length > 0) {
-                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mSubtitleTracks);
-                mSubtitles.setAdapter(adapter);
-                int current = mLibVLC.getSpuTrack();
-                mSubtitles.setSelection(current);
-                mSubtitles.setOnItemSelectedListener(mSubtitlesListener);
-                mSubtitles.setEnabled(true);
-            }
-            else {
-                mSubtitles.setEnabled(false);
-                mSubtitles.setOnItemSelectedListener(null);
-                mSubtitles.setAdapter(null);
-            }
-        }
         updateOverlayPausePlay();
     }
 
@@ -1185,6 +1158,42 @@ public class VideoPlayerActivity extends Activity {
         return time;
     }
 
+    private void setTracksAndSubtitles () {
+        if (mAudioTracks == null) {
+            mAudioTracks = mLibVLC.getAudioTrackDescription();
+            if (mAudioTracks != null && mAudioTracks.length > 1) {
+                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mAudioTracks);
+                mAudio.setAdapter(adapter);
+                int current = mLibVLC.getAudioTrack() - 1;
+                mAudio.setSelection(current);
+                mAudio.setOnItemSelectedListener(mAudioListener);
+                mAudio.setEnabled(true);
+            }
+            else {
+                mAudio.setEnabled(false);
+                mAudio.setOnItemSelectedListener(null);
+                mAudio.setAdapter(null);
+            }
+        }
+        if (mSubtitleTracks == null) {
+            mSubtitleTracks = mLibVLC.getSpuTrackDescription();
+            if (mSubtitleTracks != null && mSubtitleTracks.length > 0) {
+                ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, android.R.layout.simple_spinner_item, mSubtitleTracks);
+                mSubtitles.setAdapter(adapter);
+                int current = mLibVLC.getSpuTrack();
+                mSubtitles.setSelection(current);
+                mSubtitles.setOnItemSelectedListener(mSubtitlesListener);
+                mSubtitles.setEnabled(true);
+            }
+            else {
+                mSubtitles.setEnabled(false);
+                mSubtitles.setOnItemSelectedListener(null);
+                mSubtitles.setAdapter(null);
+            }
+        }
+    }
+
+
     /**
      *
      */
